 
 
#include "testlib.h"
#include <iostream>
#include <vector>
#include <algorithm>
#include <map>
#include <string>
 
using namespace std;
 
map<string, string> params;
 
long long get(string name, long long def = 0) {
    if (params.count(name)) {
        return stoll(params[name]);
    } else {
        return def;
    }
}
 
const int MX = 1'000'000'000;
 
vector<int> gen_numbers() {
    int n = get("n");
    int copies = get("copies", 1);
    int mn = get("minimum", 0);
    int mx = get("maximum", MX);
    int scale = get("scale", 1);
    vector<int> vec;
    while (int(vec.size()) < n) {
        int a = rnd.next(mn, mx) * scale;
        for (int t = 0; t < copies; t++) vec.push_back(a);
    }
    
    return vec;
}

void print_graph(int n, int k, vector<pair<int, int>> edges, vector<int> numbers) {
    int shuffle_vertexes = get("shuffle_vertexes", 1);
    set<pair<int, int>> eset;
    for (auto c : edges) eset.insert({min(c.first, c.second), max(c.first, c.second)});
    cout << n << " " << eset.size() << " " << k << "\n";
    edges.clear();
    for (auto c : eset) edges.push_back(c);
    
    vector<int> perm(n), inv(n);
    for (int i = 0; i < n; i++) {
        perm[i] = i;
    }
    
    if (shuffle_vertexes == 1 && n > 2) shuffle(perm.begin() + 1, perm.end() - 1);
    for (int i = 0; i < n; ++i) inv[perm[i]] = i;
    
    for (int i = 0; i < n; ++i) cout << numbers[inv[i]] << " \n"[i == n - 1];
    
    vector<pair<int, int>> new_edges;
    for (auto c : edges) {
        if (rnd.next(2)) {
            new_edges.push_back({perm[c.first], perm[c.second]});
        } else {
            new_edges.push_back({perm[c.second], perm[c.first]});
        }
    }
    
    edges = new_edges;
    shuffle(edges.begin(), edges.end());
    
    for (auto c : edges) {
        cout << c.first + 1 << " " << c.second + 1 << "\n";
    }
}
 
void test_group1() {
    int n = get("n"), m = 0, k = get("k"), degree = get("degree"), variation = get("variation"), t = get("t", 0);
    
    vector<pair<int, int>> edges;
    for (int i = 1; i < n; i++) {
        int current_degree = max(1, degree + rnd.next(-variation, variation));
        set<int> adj;
        int iter = 0;
        while (adj.size() != i && current_degree) {
            int cur = rnd.wnext(i - iter, t);
            while (adj.count(cur)) ++cur;
            adj.insert(cur);
            
            edges.push_back({cur, i});
            --current_degree;
            ++iter;
            ++m;
        }
    }

    print_graph(n, k, edges, gen_numbers());
}
 
void test_group2() {
    int n = get("n");
    int t = get("t");
    int k = get("k");

    vector<pair<int, int>> edges;
    
    /* setup parents for vertices 1..n-1 */
    for (int i = 0; i < n; i++)
        if (i > 0)
            edges.push_back({i, rnd.wnext(i, t)});
    
    print_graph(n, k, edges, gen_numbers());
}

void test_group3() {
    int n = get("n");
    int t = get("t");
    int k = get("k");
    int ruintail = get("ruintail", -1);
    int ruinlen = get("ruinlen", 2);
    vector<pair<int, int>> edges;
    
    for (int i = 0; i < n; i++)
        if (i > 0)
            edges.push_back({i, i - 1});
    
    vector<int> vals = gen_numbers();
    if (ruintail != -1) {
        for (int t = 1; t <= ruinlen; ++t) vals[ruintail + t] = MX - rnd.next(0, 100000);
        vals[ruintail] = 0;
        vals[ruintail + ruinlen + 1] = 0;
    }
    print_graph(n, k, edges, vals);
}

void test_group4() {
    int n = get("n");
    int t = get("t");
    int k = get("k");
    int prob = get("edges_prob");
    int leftright = get("sides");
    int middle = n - leftright * 2;
    int method = get("method", 1);
    
    vector<pair<int, int>> edges;
    vector<int> vals(n);
    
    for (int i = 0; i < leftright; ++i) {
        vals[i] = rnd.next(0, MX / 1000) + MX / 2;
        vals[i + leftright + middle] = rnd.next(0, MX / 1000) + MX / 2;
        
        for (int j = i + 1; j < leftright; ++j) {
            int e1 = rnd.next(0, prob), e2 = rnd.next(0, prob);
            if (e1 == 0) {
                edges.push_back({i, j});
            }
            if (e2 == 0) {
                edges.push_back({i + leftright + middle, j + leftright + middle});
            }
        }
    }
    
    int s0 = 0, s1 = MX;
    for (int i = leftright; i <= leftright + middle; ++i) {
        edges.push_back({i - 1, i});
        if (i != leftright + middle && method == 1) vals[i] = (i % 2) * MX;
        if (i != leftright + middle && method == 2) vals[i] = rnd.next(0, MX);
    }

    print_graph(n, k, edges, vals);
}
 
void test_group5() {
    int n = get("n"), k = get("k");
    
    vector<pair<int, int>> edges;
    vector<int> vals(n);
    for (int i = 2; i + 1 < n; ++i) {
        edges.push_back({i, i - 2});
    }
    edges.push_back({0, 1});
    edges.push_back({n - 2, n - 1});
    edges.push_back({n - 3, n - 1});
    vals[0] = vals[1] = vals[2] = 0;
    vals[3] = vals[4] = vals[5] = vals[6] = MX;
    vals[7] = vals[8] = 0;
    
    for (int i = 7; i < n; ++i) vals[i] = rnd.next(0, MX / 2500);
    
    print_graph(n, k, edges, vals);
}
 
/*
void test_group4() {
    int n = get("n");
    cout << n << "\n";
    vector<int> perm(n);
    for (int i = 0; i < n; i++)
        perm[i] = i;
    shuffle(perm.begin() + 1, perm.end());
    
    vector<pair<int,int> > edges;
    gen_numbers();
 
    for (int i = 1; i < n; i++) {
        edges.push_back({perm[i], perm[i - 1]});
    }
    
    shuffle(edges.begin(), edges.end());
    
    for (int i = 0; i + 1 < n; i++) {
        printf("%d %d\n", edges[i].first + 1, edges[i].second + 1);
    }
}
*/
 
 
int main(int argc, char* argv[])
{
    registerGen(argc, argv, 1);
    for (int i = 1; i < argc; i++) {
        string s = string(argv[i]);
        if (s.find("=") != string::npos) {
            auto pos = s.find("=");
            params[s.substr(0, pos)] = s.substr(pos + 1);
        } else {
            params[s] = "";
        }
    }
 
    int mode = get("mode");
    
    if (mode == 1) {
        test_group1();
    }
    
    if (mode == 2) {
        test_group2();
    }
    
    if (mode == 3) {
        test_group3();
    }
    
    if (mode == 4) {
        test_group4();
    }
    
    if (mode == 5) {
        test_group5();
    }
 
    return 0;
}
 
