# include "testlib.h"
# include <string>
# include <vector>
 
using namespace std;
 
const int MAXN = 2000, MAXM = 2000, MAXK = 10, MAXW = 1'000'000'000;
 
vector<int> gr[MAXN];
bool used[MAXN];
 
void dfs(int v)
{
    used[v] = 1;
 
    for (auto to : gr[v])
    {
        if (used[to]) continue;
        dfs(to);
    }
}
 
int main(int argc, char* argv[])
{
    registerValidation(argc, argv);
    
    string group = validator.group();
    int n = inf.readInt(2, MAXN, "n");
    inf.readSpace();
    int m = inf.readInt(1, MAXM, "m");
    inf.readSpace();
    int k = inf.readInt(0, MAXK, "k");
    inf.readEoln();
    
    vector<int> vals;
    for (int i = 0; i < n; i++)
    {
        int h = inf.readInt(0, MAXW, "a");
        vals.push_back(h);
        if (i != n - 1) inf.readSpace();
    }
 
    inf.readEoln();
    
    set<pair<int, int>> st;
    for (int i = 0; i < m; i++)
    {
        int u = inf.readInt(1, n, "u");
        inf.readSpace();
        int v = inf.readInt(1, n, "v");
        ensuref(u != v, "There must be no loops");
        if (u > v) swap(u, v);
        st.insert({u, v});
        inf.readEoln();
 
        gr[u - 1].push_back(v - 1);
        gr[v - 1].push_back(u - 1);
    }
    ensuref(st.size() == m, "There must be no multiple edges");
 
    dfs(0);
    for (int i = 0; i < n; ++i) {
        ensuref(used[i], "Graph must be connected.");
    }
    
    if (group.find("tree") != std::string::npos) {
        ensuref(m == n - 1, "Graph must form a tree");
    }
    if (group.find("bamboo") != std::string::npos) {
        bool flag = (st.size() == n - 1);
        for (int i = 0; i + 1 < n; ++i) {
            if (!st.count(make_pair(i + 1, i + 2))) flag = 0;
        }
        ensuref(flag, "Graph must form a bamboo");
    }
    if (group.find("k0") != std::string::npos) {
        ensuref(k == 0, "k must be equal to 0");
    }
    if (group.find("small") != std::string::npos) {
        bool flag = true;
        for (auto c : vals) flag &= (c <= 20);
        ensuref(flag, "weights must not exceed 20");
    }
 
    inf.readEof();
}
